/*
 * Placeholder PetaLinux user application.
 *
 * Replace this with your application code
 */
#include <fcntl.h>
#include <stdio.h>
#include <linux/i2c-dev.h>
#include "audio.h"
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <termios.h>

#define ZED_MAP_SIZE 0x1000
#define I2S_DATA_RX_L_REG 0x00 
#define I2S_DATA_RX_R_REG 0x04 
#define I2S_DATA_TX_L_REG 0x08 
#define I2S_DATA_TX_R_REG 0x0c 
#define I2S_STATUS_REG 0x10 

#define I2C_SLAVE 0x0703

int Fdiic;

int kbhit(void)
{
	struct termios oldt, newt;
	int ch;
	int oldf;

	tcgetattr(STDIN_FILENO, &oldt);
	newt = oldt;
	newt.c_lflag &= ~(ICANON | ECHO);
	tcsetattr(STDIN_FILENO, TCSANOW, &newt);
	oldf = fcntl(STDIN_FILENO, F_GETFL, 0);
	fcntl(STDIN_FILENO, F_SETFL, oldf | O_NONBLOCK);

	ch = getchar();

	tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
	fcntl(STDIN_FILENO, F_SETFL, oldf);
	if(ch != EOF)
	{
		ungetc(ch, stdin);
		return 1;
	}
	return 0;
}

/******************************************************************************
 * Function to write one byte (8-bits) to one of the registers from the audio
 * controller.
 *
 * @param	u8RegAddr is the LSB part of the register address (0x40xx).
 * @param	u8Data is the data byte to write.
 *
 * @return	none.
 *****************************************************************************/
void AudioWriteToReg(unsigned char u8RegAddr, unsigned char u8Data) {

	unsigned char u8TxData[3];
	int BytesWritten; 

	u8TxData[0] = 0x40;
	u8TxData[1] = u8RegAddr;
	u8TxData[2] = u8Data;

	BytesWritten = write(Fdiic, u8TxData, 3);
	if (BytesWritten != 3)
		printf("Error in writing %x at regaddr %x\n",u8Data, u8RegAddr);
}

/******************************************************************************
 * Configures audio codes's various mixers, ADC's, DAC's, and amplifiers to
 * accept stereo input from line in and push stereo output to line out
 *
 * @param	none.
 *
 * @return	none.
 *****************************************************************************/
void AudioConfigureJacks()
{
	AudioWriteToReg(R4_RECORD_MIXER_LEFT_CONTROL_0, 0x01); //enable mixer 1
	AudioWriteToReg(R5_RECORD_MIXER_LEFT_CONTROL_1, 0x07); //unmute Left channel of line in into mxr 1 and set gain to 6 db
	AudioWriteToReg(R6_RECORD_MIXER_RIGHT_CONTROL_0, 0x01); //enable mixer 2
	AudioWriteToReg(R7_RECORD_MIXER_RIGHT_CONTROL_1, 0x07); //unmute Right channel of line in into mxr 2 and set gain to 6 db
	AudioWriteToReg(R19_ADC_CONTROL, 0x13); //enable ADCs

	AudioWriteToReg(R22_PLAYBACK_MIXER_LEFT_CONTROL_0, 0x21); //unmute Left DAC into Mxr 3; enable mxr 3
	AudioWriteToReg(R24_PLAYBACK_MIXER_RIGHT_CONTROL_0, 0x41); //unmute Right DAC into Mxr4; enable mxr 4
	AudioWriteToReg(R26_PLAYBACK_LR_MIXER_LEFT_LINE_OUTPUT_CONTROL, 0x05); //unmute Mxr3 into Mxr5 and set gain to 6db; enable mxr 5
	AudioWriteToReg(R27_PLAYBACK_LR_MIXER_RIGHT_LINE_OUTPUT_CONTROL, 0x11); //unmute Mxr4 into Mxr6 and set gain to 6db; enable mxr 6
	AudioWriteToReg(R29_PLAYBACK_HEADPHONE_LEFT_VOLUME_CONTROL, 0x00);//Mute Left channel of HP port (LHP)
	AudioWriteToReg(R30_PLAYBACK_HEADPHONE_RIGHT_VOLUME_CONTROL, 0x00); //Mute Right channel of HP port (LHP)
	AudioWriteToReg(R31_PLAYBACK_LINE_OUTPUT_LEFT_VOLUME_CONTROL, 0xE6); //set LOUT volume (0db); unmute left channel of Line out port; set Line out port to line out mode
	AudioWriteToReg(R32_PLAYBACK_LINE_OUTPUT_RIGHT_VOLUME_CONTROL, 0xE6); // set ROUT volume (0db); unmute right channel of Line out port; set Line out port to line out mode
	AudioWriteToReg(R35_PLAYBACK_POWER_MANAGEMENT, 0x03); //enable left and right channel playback (not sure exactly what this does...)
	AudioWriteToReg(R36_DAC_CONTROL_0, 0x03); //enable both DACs

	AudioWriteToReg(R58_SERIAL_INPUT_ROUTE_CONTROL, 0x01); //Connect I2S serial port output (SDATA_O) to DACs
	AudioWriteToReg(R59_SERIAL_OUTPUT_ROUTE_CONTROL, 0x01); //connect I2S serial port input (SDATA_I) to ADCs

	AudioWriteToReg(R65_CLOCK_ENABLE_0, 0x7F); //Enable clocks
	AudioWriteToReg(R66_CLOCK_ENABLE_1, 0x03); //Enable rest of clocks
}


/******************************************************************************
 * Configures audio codes's internal PLL. With MCLK = 8 MHz it configures the
 * PLL for a VCO frequency = 49.152 MHz, and an audio sample rate of 48 KHz.
 *
 * @param	none.
 *
 * @return	none.
 *****************************************************************************/
void AudioPllConfig() {

	unsigned char u8TxData[8], u8RxData[6];
	int Status;
	int BytesWritten, BytesRead;

	Fdiic = open("/dev/i2c-0", O_RDWR); 
	if(Fdiic < 0) {
		printf("\nError initializing IIC");
		return -1;
	}

	Status = ioctl(Fdiic, I2C_SLAVE, IIC_SLAVE_ADDR>>1); 
	if(Status < 0) {
		printf("\n Unable to set the address\n");
		return -1;
	}
	
	AudioWriteToReg(R0_CLOCK_CONTROL, 0x0E); // was 0x01

	// Write 6 bytes to R1
	u8TxData[0] = 0x40;
	u8TxData[1] = 0x02;
	u8TxData[2] = 0x00; // byte 1
	u8TxData[3] = 0x7d; // byte 2
	u8TxData[4] = 0x00; // byte 3
	u8TxData[5] = 0x12; // byte 4
	u8TxData[6] = 0x31; // byte 5
	u8TxData[7] = 0x01; // byte 6


	BytesWritten = write(Fdiic, u8TxData, 8);
	if (BytesWritten != 8)
		printf("Bytes sent=8, Bytes written=%d\n", BytesWritten);

	// Poll PLL Lock bit
	u8TxData[0] = 0x40;
	u8TxData[1] = 0x02;

	do {
		BytesWritten = write(Fdiic, u8TxData, 2);
		BytesRead = read(Fdiic, u8RxData, 6);
	}
	while((u8RxData[5] & 0x02) == 0);

	AudioWriteToReg(R0_CLOCK_CONTROL, 0x0F);//COREN
}

void bypass_input(void *zed_ptr, void *gpio_ptr)
{
	unsigned u32DataL, u32DataR;

	while (1)
	{
		while(1) { //wait for RX data to become available
			if(*((unsigned *)(zed_ptr + I2S_STATUS_REG)))	
				break;
			else 
				continue;
		}	
		*((unsigned *)(zed_ptr + I2S_STATUS_REG)) = 0x1; //Clear data rdy bit

		u32DataL = *((unsigned *)(zed_ptr + I2S_DATA_RX_L_REG));
		u32DataR = *((unsigned *)(zed_ptr + I2S_DATA_RX_R_REG));

		if((*((unsigned *)(gpio_ptr + 8)))&0x01) { // through mode
			*((unsigned *)(zed_ptr + I2S_DATA_TX_L_REG)) = u32DataL;
			*((unsigned *)(zed_ptr + I2S_DATA_TX_R_REG)) = u32DataR;
		}
		else // mute mode
		{
			*((unsigned *)(zed_ptr + I2S_DATA_TX_L_REG)) = 0;
			*((unsigned *)(zed_ptr + I2S_DATA_TX_R_REG)) = 0;
		}
		if(kbhit()) {
			printf(" Done. Exiting...\n"); 
			break; 
		}
	}
}


int main(int argc, char *argv[])
{
	
	int i;
	
	/* for Audio Control */
	int zed_fd;
	char *zed_uiobf="/dev/uio3";
	void *zed_ptr;

	/* for GPIO Control */
	int gpio_fd, sel;
	char *gpio_uiobf="/dev/uio0";
	void *gpio_ptr;

	/* GPIO core */
	gpio_fd = open(gpio_uiobf, O_RDWR);
	if (gpio_fd < 1) {
		printf("Invalid UIO device file:%s.\n", gpio_uiobf);
		return -1;
	}

	/* mmap the UIO devices */
	gpio_ptr = mmap(NULL, ZED_MAP_SIZE, PROT_READ|PROT_WRITE, MAP_SHARED, gpio_fd, 0);

	/* set directions, channel 0 => output, channel 1 => input */
	*((unsigned *)(gpio_ptr + 4)) = 0x0;	// output direction
	*((unsigned *)(gpio_ptr + 0)) = 0x3;	// output data

	/* set SW0 input direction*/
	*((unsigned *)(gpio_ptr + 0xC)) = 0x1;	// input direction

	//Configure the Audio Codec's PLL
	AudioPllConfig();

	//Configure the Line in and Line out ports.
	//Call LineInLineOutConfig() for a configuration that
	//enables the HP jack too.
	AudioConfigureJacks();

	printf("Audio CODEC initialized\n");

	/* audio core */
	zed_fd = open(zed_uiobf, O_RDWR);
	if (zed_fd < 1) {
		printf("Invalid UIO device file:%s.\n", zed_uiobf);
		return -1;
	}

	/* mmap the UIO devices */
	zed_ptr = mmap(NULL, ZED_MAP_SIZE, PROT_READ|PROT_WRITE, MAP_SHARED, zed_fd, 0);

	printf("Put SW0 to UP position for Through mode, DN position to Mute mode\n");
	printf("Press any key to terminate the program\n");

	bypass_input(zed_ptr, gpio_ptr); 

	return 0;
}


